#!/bin/sh

# ------------------------------------------------------------------------
# 
#  Oracle configuration auditing script by DCIT, a.s.
#  Author: Karel Miko (miko@dcit.cz)
#  Notes:
#     *) should be executed in OS environment on DB server
#     *) should be run under OS user "oracle"
#     *) requires existing directory ./out/ for output data  
#  
#  ------------------------------------------------------------------------

SCRIPT_VERSION="1.2"

OUTDIR="./out"
OUTPREF="$OUTDIR/oracfgcheck_os_`date +%y%m%d_%H%M%S`"
OUTFILE="$OUTPREF.txt"

echo
echo "STARTED oracheck-unix.sh"
echo "(outfile=$OUTFILE, script version $SCRIPT_VERSION)"
echo

if [ -d "$OUTDIR" ]; then
  echo "Started oracheck-unix.sh" > "$OUTFILE"
  echo "(outfile=$OUTFILE, script version $SCRIPT_VERSION)" >> "$OUTFILE"

	echo "// --------------------------" | tee -a "$OUTFILE"
	echo "// [01] OS - date" | tee -a "$OUTFILE"
	date 2>&1 | tee -a "$OUTFILE" 
  echo | tee -a "$OUTFILE"

	echo "// --------------------------" | tee -a "$OUTFILE"
	echo "// [02] OS - uname" | tee -a "$OUTFILE"
	uname -a 2>&1 | tee -a "$OUTFILE"
  echo | tee -a "$OUTFILE"

	echo "// --------------------------" | tee -a "$OUTFILE"
	echo "// [03] OS - id" | tee -a "$OUTFILE"
	echo "// IMPORTANT: we need to run as 'oracle'" | tee -a "$OUTFILE"
	id 2>&1 | tee -a "$OUTFILE" 
  echo | tee -a "$OUTFILE"
  
	echo "// --------------------------" | tee -a "$OUTFILE"
	echo "// [04] OS - groups with sysdba/sysop mapping" | tee -a "$OUTFILE"
	grep define.SS $ORACLE_HOME/rdbms/lib/config.c 2>&1 | tee -a "$OUTFILE" 
  echo | tee -a "$OUTFILE"
  
  echo "// --------------------------" | tee -a "$OUTFILE"
	echo "// [05] OS - important users" | tee -a "$OUTFILE"
	grep -e "^dba" -e "^ora" /etc/passwd 2>&1 | tee -a "$OUTFILE" 
  echo | tee -a "$OUTFILE"
  
  echo "// --------------------------" | tee -a "$OUTFILE"
	echo "// [06] OS - important groups" | tee -a "$OUTFILE"
	grep -e "^dba" -e "^ora" -e "^oinstall" /etc/group 2>&1 | tee -a "$OUTFILE" 
  echo | tee -a "$OUTFILE"
    
	echo "// --------------------------" | tee -a "$OUTFILE"
	echo "// [07] OS - $ORACLE_HOME env variable" | tee -a "$OUTFILE"
	if [ -z "$ORACLE_HOME" ];then
		echo "WARNING: variable $ORACLE_HOME not set" | tee -a "$OUTFILE"
		echo "// IMPORTANT: we need ORACLE environment variables" | tee -a "$OUTFILE" 
	else
		echo "ORACLE_HOME=$ORACLE_HOME" | tee -a "$OUTFILE"
	fi
  echo | tee -a "$OUTFILE"
	
	echo "// --------------------------" | tee -a "$OUTFILE"
	echo "// [08] OS - environment ORA*, PATH, HOME, HOSTNAME, SHELL, LD*" | tee -a "$OUTFILE"
	set | grep -e \^ORA -e \^PATH -e \^HOME -e \^HOSTNAME -e \^SHELL -e \^LD 2>&1 | tee -a "$OUTFILE"
  echo | tee -a "$OUTFILE"

	echo "// --------------------------" | tee -a "$OUTFILE"
	echo "// [09] OS - crontab jobs" | tee -a "$OUTFILE"
	echo "// check ACL of all scripts - no access for group and world" | tee -a "$OUTFILE"
	crontab -l 2>&1 | tee -a "$OUTFILE"
  echo | tee -a "$OUTFILE"

	echo "// --------------------------" | tee -a "$OUTFILE"
	echo "// [10] OS - at jobs" | tee -a "$OUTFILE"
	echo "// check ACL of all scripts - no access for group and world" | tee -a "$OUTFILE"
	at -l 2>&1 | tee -a "$OUTFILE"
  echo | tee -a "$OUTFILE"

	echo "// --------------------------" | tee -a "$OUTFILE"
	echo "// [11] OS - oracle processes" | tee -a "$OUTFILE"
	echo "// output='$OUTPREF-oraprocs.txt'" | tee -a "$OUTFILE"
	ps -ef 2>&1 | grep -i "ora_" 2>&1 | grep -v grep 2>&1 > "$OUTPREF-oraprocs.txt"	
  echo | tee -a "$OUTFILE"

	echo "// --------------------------" | tee -a "$OUTFILE"
	echo "// [12] OS - netstat" | tee -a "$OUTFILE"
	echo "// output='$OUTPREF-tcplisten-list1.txt'" | tee -a "$OUTFILE"
	echo "// output='$OUTPREF-tcplisten-list2.txt'" | tee -a "$OUTFILE"
	netstat -an 2>&1 | grep LISTEN > "$OUTPREF-tcplisten-list1.txt"
	netstat -anp 2>&1 | grep LISTEN > "$OUTPREF-tcplisten-list2.txt"
  echo | tee -a "$OUTFILE"

	echo "// --------------------------" | tee -a "$OUTFILE"
	echo "// [13] ORACLE - running instances" | tee -a "$OUTFILE"
	echo "// see also '$OUTPREF-instances.txt'" | tee -a "$OUTFILE"
	ps -ef | grep -i "ora_pmon" | sed "s,^.*ora_pmon_,," | sort | uniq | tee -a "$OUTPREF-instances.txt" | tee -a "$OUTFILE"
  echo | tee -a "$OUTFILE"

	echo "// --------------------------" | tee -a "$OUTFILE"
	echo "// [14] ORACLE - listener aliases" | tee -a "$OUTFILE"
	echo "// see also '$OUTPREF-listeners.txt'" | tee -a "$OUTFILE"
	ps -efw | grep -i "/tnslsnr" | sed "s,^.*/tnslsnr ,," | sed "s,\([^ ]*\).*,\1," | sort | uniq | tee -a "$OUTPREF-listeners.txt" | tee -a "$OUTFILE"
  echo | tee -a "$OUTFILE"

	echo "// --------------------------" | tee -a "$OUTFILE"
	echo "// [15] ORACLE - opatch" | tee -a "$OUTFILE"
	if [ -f "$ORACLE_HOME/OPatch/opatch" ]; then 
		"$ORACLE_HOME/OPatch/opatch" lsinventory > "$OUTPREF-opatch-lsinventory.txt" 2>&1
		echo "// output='$OUTPREF-opatch-lsinventory.txt'" | tee -a "$OUTFILE"
		"$ORACLE_HOME/OPatch/opatch" lsinventory -detail > "$OUTPREF-opatch-lsinventory-detail.txt" 2>&1
		echo "// output='$OUTPREF-opatch-lsinventory-detail.txt'" | tee -a "$OUTFILE"
		"$ORACLE_HOME/OPatch/opatch" lsinventory -all > "$OUTPREF-opatch-lsinventory-all.txt" 2>&1
		echo "// output='$OUTPREF-opatch-lsinventory-all.txt'" | tee -a "$OUTFILE"
	else
		echo "WARNING: opatch not found" | tee -a "$OUTFILE"
	fi
  echo | tee -a "$OUTFILE"

	echo "// --------------------------" | tee -a "$OUTFILE"
	echo "// [16] ORACLE configuration files" | tee -a "$OUTFILE"
	if [ -d "$ORACLE_HOME/network/admin/" ]; then
		cp "$ORACLE_HOME/network/admin/listener.ora" "$OUTPREF-listener.ora.txt"
		echo "// output='$OUTPREF-listener.ora.txt'" | tee -a "$OUTFILE"
		cp "$ORACLE_HOME/network/admin/tnsnames.ora" "$OUTPREF-tnsnames.ora.txt"
		echo "// output='$OUTPREF-tnsnames.ora.txt'" | tee -a "$OUTFILE"
		cp "$ORACLE_HOME/network/admin/sqlnet.ora" "$OUTPREF-sqlnet.ora.txt"
		echo "// output='$OUTPREF-sqlnet.ora.txt'" | tee -a "$OUTFILE"
	else
		echo "WARNING: configuration files *.ora not found" | tee -a "$OUTFILE"
	fi 	
  echo | tee -a "$OUTFILE"

	echo "// --------------------------" | tee -a "$OUTFILE"
	echo "// [17] LISTENER - default listener" | tee -a "$OUTFILE"
	echo "// output='$OUTPREF-lsnr-status1.txt'" | tee -a "$OUTFILE"
	echo "// output='$OUTPREF-lsnr-services1.txt'" | tee -a "$OUTFILE"
	lsnrctl status LISTENER > "$OUTPREF-lsnr-status1.txt" 2>&1
	lsnrctl services LISTENER > "$OUTPREF-lsnr-services1.txt" 2>&1
  echo | tee -a "$OUTFILE"
	
	echo "// --------------------------" | tee -a "$OUTFILE"
	echo "// [18] LISTENER - dumping detected aliases" | tee -a "$OUTFILE"
	cat "$OUTPREF-listeners.txt" | sort | uniq | while read LSN
	do
		echo "// output='$OUTPREF-lsnr-$LSN-status.txt'" | tee -a "$OUTFILE"
 		echo "// output='$OUTPREF-lsnr-$LSN-services.txt'" | tee -a "$OUTFILE"
	 	lsnrctl status $LSN > "$OUTPREF-lsnr-$LSN-status.txt" 2>&1
	 	lsnrctl services $LSN > "$OUTPREF-lsnr-$LSN-services.txt" 2>&1
	done
	echo | tee -a "$OUTFILE"

	echo "// --------------------------" | tee -a "$OUTFILE"
	echo "// [19] FILESYSTEM - files/dirs with non-std ACL - standard = world-notwritable, group-notwritable" | tee -a "$OUTFILE"
  echo "// should return an empty result" | tee -a "$OUTFILE"
  echo "// see also '$OUTPREF-fs-nonstd.txt'" | tee -a "$OUTFILE"
  find $ORACLE_HOME -perm +022 ! -type l -exec ls -ld {} \;  2>&1 | tee -a "$OUTPREF-fs-nonstd.txt" | tee -a "$OUTFILE"
  echo | tee -a "$OUTFILE"

	echo "// --------------------------" | tee -a "$OUTFILE"
	echo "// [20] FILESYSTEM - files that should not be accessible by 'world'" | tee -a "$OUTFILE"
  echo "// check for '-rw-r----- [oracle:oinstall]'" | tee -a "$OUTFILE"
  ls -ld $ORACLE_HOME/dbs/orapw* 2>&1 | tee -a "$OUTFILE"
  ls -ld $ORACLE_HOME/network/admin/*ora  2>&1 | tee -a "$OUTFILE"
  echo | tee -a "$OUTFILE"

	echo "// --------------------------" | tee -a "$OUTFILE"
	echo "// [21] FILESYSTEM - world writable files/dirs in ORACLE_HOME" | tee -a "$OUTFILE"
  echo "// should return an empty result" | tee -a "$OUTFILE"
  echo "// see also '$OUTPREF-fs-orahome-ww.txt'" | tee -a "$OUTFILE"
	find $ORACLE_HOME -perm -002 ! -type l -exec ls -ld {} \;  2>&1 | tee -a "$OUTPREF-fs-orahome-ww.txt"| tee -a "$OUTFILE"
  echo | tee -a "$OUTFILE"

	echo "// --------------------------" | tee -a "$OUTFILE"
	echo "// [22] FILESYSTEM - world writable files/dirs in the whole filesystem" | tee -a "$OUTFILE"
	echo "// output='$OUTPREF-fs-all-ww.txt'" | tee -a "$OUTFILE"
	find / -perm -002 ! -type l -exec ls -ld {} \; > "$OUTPREF-fs-all-ww.txt" 2>&1
  echo | tee -a "$OUTFILE"

	echo "// --------------------------" | tee -a "$OUTFILE"
	echo "// [23] FILESYSTEM - files in ORACLE_HOME with owner different from oracle:oinstall" | tee -a "$OUTFILE"
	echo "// the following exceptions are acceptable" | tee -a "$OUTFILE"
	echo "//   -r-sr-s---   root oinstall $ORACLE_HOME/bin/oradism" | tee -a "$OUTFILE"
	echo "//   -rwsr-s---   root oinstall $ORACLE_HOME/bin/nmb" | tee -a "$OUTFILE"
	echo "//   -rwsr-s---   root oinstall $ORACLE_HOME/bin/nmo" | tee -a "$OUTFILE"
	echo "//   -rwsr-x---   root oinstall $ORACLE_HOME/bin/extjob" | tee -a "$OUTFILE"
	echo "//   -rw-r-----   root oinstall $ORACLE_HOME/rdbms/admin/externaljob.ora" | tee -a "$OUTFILE"
	echo "// see also '$OUTPREF-fs-orahome-ownr.txt'" | tee -a "$OUTFILE"
	find $ORACLE_HOME \( ! -group oinstall -o ! -user oracle \) -exec ls -ld {} \;  2>&1 | tee -a "$OUTPREF-fs-orahome-ownr.txt" | tee -a "$OUTFILE"
  echo | tee -a "$OUTFILE"

	echo "// --------------------------" | tee -a "$OUTFILE"
	echo "// [24] FILESYSTEM - files in ORACLE_HOME with setuid/gid bit" | tee -a "$OUTFILE"
	echo "// the following exceptions are acceptable" | tee -a "$OUTFILE"
	echo "//   -r-sr-s---   root oinstall $ORACLE_HOME/bin/oradism" | tee -a "$OUTFILE"
	echo "//   -rwsr-s---   root oinstall $ORACLE_HOME/bin/nmb" | tee -a "$OUTFILE"
	echo "//   -rwsr-s---   root oinstall $ORACLE_HOME/bin/nmo" | tee -a "$OUTFILE"
	echo "//   -rwsr-sr-x   root oinstall $ORACLE_HOME/bin/oracle" | tee -a "$OUTFILE"
	echo "//   -rwsr-sr-x   root oinstall $ORACLE_HOME/bin/emtgtctl2" | tee -a "$OUTFILE"
	echo "//   -rwsr-x---   root oinstall $ORACLE_HOME/bin/extjob" | tee -a "$OUTFILE"
	echo "// see also '$OUTPREF-fs-orahome-sbit.txt'" | tee -a "$OUTFILE"
	find $ORACLE_HOME -perm +6000 -exec ls -ld {} \;  2>&1 | tee -a "$OUTPREF-fs-orahome-sbit.txt"| tee -a "$OUTFILE"
  echo | tee -a "$OUTFILE"

	echo "// --------------------------" | tee -a "$OUTFILE"
	echo "// [25] FILESYSTEM - files *O (backup copies) in ORACLE_HOME with setuid/gid bit" | tee -a "$OUTFILE"
  echo "// should return an empty result" | tee -a "$OUTFILE"
  echo "// see also '$OUTPREF-fs-orahome-Os.txt'" | tee -a "$OUTFILE"
	find $ORACLE_HOME -name "*O" -perm +6000 -exec ls -ld {} \;  2>&1 | tee -a "$OUTPREF-fs-orahome-Os.txt"| tee -a "$OUTFILE"
  echo | tee -a "$OUTFILE"

	echo "FINISHED" | tee -a "$OUTFILE"
	echo "See results in '$OUTPREF*'" | tee -a "$OUTFILE"
  echo | tee -a "$OUTFILE"
else
  echo "ERROR: directory $OUTDIR does not exist"
	exit
fi
