-- ------------------------------------------------------------------------
--
-- Oracle configuration auditing script by DCIT, a.s.
-- Author: Karel Miko (miko@dcit.cz)
-- Notes:
--    *) should be executed through Sql*Plus
--    *) requires DBA privileges 
--    *) tested on Oracle 10gR2
--    *) does not reveal any password hashes (from USER$, DBA_USERS)
-- 
-- ------------------------------------------------------------------------

define my_script_version = '1.2'

set echo off
set linesize 30000
set pagesize 0      
set trimspool on
set feedback off
set termout on

clear breaks
clear computes
clear columns

-- define output filename
alter session set nls_date_format = 'YYMMDD_HH24MISS';
define my_outfile='out/oracfgcheck_&_CONNECT_IDENTIFIER._&_DATE..txt'

-- set proper time-date format
alter session set nls_date_format = 'DD.MM.YYYY HH24:MI';

spool &my_outfile
timing start "Script oracheck.sql"

-- ----------------------------------------------

prompt // ORACLE CONFIGURATION CHECK STARTED
prompt // (script version: &my_script_version)
set pagesize 0
select 'Current date: ' || to_char(sysdate,'DD.MM.YYYY HH24:MI') from dual;
select 'Current DB instance name: ' || INSTANCE_NAME from v$instance;
select 'Current DB server name: ' || HOST_NAME from v$instance;
select 'Current DB user: ' || user from dual;
select 'Current DB oracle version: ' || VERSION from v$instance;

set pagesize 50000
set feedback on
set termout off

-- ----------------------------------------------

prompt
set termout on
prompt // [01] DB INSTANCE DETAIL (1)
set termout off
prompt // pouze informativni dotaz
prompt //
col uptime format A15
select instance_name,version,status,STARTUP_TIME Startup,trunc(SYSDATE-(STARTUP_TIME),1)||' days' Uptime from sys.v$instance;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [02] DB INSTANCE DETAIL (2)
set termout off
prompt // CHECK: production environment of OLTP systems should have ARCHIVEMOD enabled  
prompt //        disabled ARCHIVEMOD is acceptable only in OLAP/DWH applications or in a test/dev environment
prompt //
col PLATFORM_NAME format A40
select NAME,CREATED,LOG_MODE,PLATFORM_NAME from sys.v$database;
clear col

-- ----------------------------------------------
prompt
set termout on
prompt // [03] CURRENTLY CONNECTED USERS - session count
set termout off
prompt // summary - number of sessions per logged in user - just info  
prompt //
col OSUSER format A15
col DBUSER format A15
col NAME format A40
col NAME format A40
col NAME format A40
select username,osuser,count(*) session_count
from v$session
where username is not null
group by username, osuser
order by username, osuser;
clear col

-- ----------------------------------------------
prompt
set termout on
prompt // [04] CURRENTLY CONNECTED USERS - CPU usage statistics
set termout off
prompt // statistics - CPU usage per currently connected user - just info  
prompt //
select nvl(ss.username,'<oracle server process>') username,ss.osuser,sum(se.value) cpu_usage
from v$session ss, v$sesstat se, v$statname sn
where se.statistic# = sn.statistic# and name like '%CPU used by this session%' and se.sid = ss.sid
group by username,osuser
order by username;
clear col

-- ----------------------------------------------
prompt
set termout on
prompt // [05] CURRENTLY CONNECTED USERS - details
set termout off
prompt // details about all currently connected users
prompt // CHECK: review whether special system accounts (schema owners etc.) are not used for day-to-day operations 
prompt //
col OSUSER format A15
col DBUSER format A15
col NAME format A40
col NAME format A40
col NAME format A40
select all v$session.osuser osuser, v$session.username dbuser, v$session.terminal terminal, v$session.program program, v$process.spid pid
from 	v$process, v$session 
where v$process.addr=v$session.paddr
order by 1,2;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [06] DB VERSION
set termout off
prompt // CHECK: review whether installed patchset is supported by Pracle (supported = quarterly CPUs are released)
prompt //        details about supported versions, patchsets, CPUs, ... see at metalink.oracle.com
prompt //
col PRODUCT format A50
col VERSION format A30
col STATUS format A20
select * from product_component_version;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [07] DB PATCH INFO
set termout off
prompt // CHECK: review whether current Critical Patch Update (CPU) is installed - CPUs are released quaterly (Jan/Apr/Jul/Oct)
prompt // BEWARE !!! this method of getting patch information is not reliable, more precise info can be aquired
prompt //            directly from DB server with "$ORACLE_HOME/OPatch/opatch lsinventory" 
prompt //
col ACTION_TIME format A35
col VERSION format A20
col COMMENTS format A20
select action_time, version, id, comments from dba_registry_history;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [08] DB OPTIONS INSTALLED (selected items)
set termout off
prompt // just info
prompt //
col VALUE format A15
select parameter, value from v$option
where lower(parameter) in ('fine-grained access control','fine-grained auditing','enterprise user security','oracle data guard','oracle label security','oracle database vault','backup encryption','transparent data encryption','java','proxy authentication/authorization'); 
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [09] DB LICENCE INFO
set termout off
prompt // just info
prompt //
select sessions_max,sessions_warning,sessions_current,sessions_highwater,users_max from v_$license;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [10] DB SGA INFO
set termout off
prompt // just info
prompt //
select name,value from v$sga;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [11] DB PARAMETERS - files/directories:
set termout off
prompt // CHECK: review whether files/dirs permissions are set to "drwxr-x---" or "-rwxr-x---" (or even more strict)
prompt //        owner should be always "oracle:oinstall"
prompt // Note: values of some parameters can be empty
prompt //
col name format A30
col value format A88
select name, value from v$parameter where
lower(NAME) in ('background_dump_dest','user_dump_dest','core_dump_dest','spfile','ifile','db_recovery_file_dest')
or (upper(name) like 'LOG_ARCHIVE_DEST%' and upper(name) not like 'LOG_ARCHIVE_DEST_STATE%') or upper(name)='LOG_ARCHIVE_DUPLEX_DEST'; 
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [12] DB PARAMETERS - others:
set termout off
prompt // CHECK: review the following recommended values
prompt // - os_roles = FALSE
prompt // - remote_os_authent = FALSE
prompt // - remote_os_roles = FALSE
prompt // - O7_DICTIONARY_ACCESSIBILITY = FALSE
prompt // - remote_login_passwordfile = NONE (perhaps less trict EXCLUSIVE)
prompt // - utl_file_dir = <empty value>
prompt // - os_authent_prefix = <empty value> (default "OPS$" is not critical)
prompt // - sql92_security = TRUE
prompt //
col name format A33
col value format A33
select name, value from v$parameter where lower(NAME) in ('os_roles', 'remote_os_authent', 'remote_os_roles', 'o7_dictionary_accessibility', 'remote_login_passwordfile', 'os_authent_prefix', 'sql92_security', 'utl_file_dir');
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [13] DATAFILES, CONTROLFILES, REDOLOGFILES:
set termout off
prompt // CHECK: review permissions - all files should have "-rw-r-----" (owner "oracle:oinstall")
prompt //
col name format A105
col member format A105
col type format A13
select 'Controlfile' Type , name from v$controlfile
union all select 'Logfile' Type, member from v$logfile
union all select 'Datafile' Type, name from v$datafile;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [14] DATAFILE - detail info
set termout off
prompt // just info
prompt //
col DATAFILE format A80
col TABLESPACE format A15
col STATUS format A15
col USED format A10
select df.file_name datafile, df.tablespace_name tablespace, round(df.bytes/(1024*1024),2) size_mb, df.status, round(100*(ex.bytes/df.bytes),0) || '%' used
from dba_data_files df left join ( select file_id, sum(bytes) bytes from dba_extents group by file_id order by file_id ) ex 
on df.file_id = ex.file_id;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [15] DIRECTORY OBJECTS:
set termout off
prompt // CHECK: filesystem directory mapping into oracle should be defined just for the items explicitly required by application 
prompt //        especially risky is to map dirs like "/", "/oracle/home" or even system directories like "/etc" 
prompt //        when possible do not crete directory objects for any subdirs within $ORACLE_HOME, Datafiles dirs, Controlfiles dirs, ...  
prompt // CHECK: review filesystem permissions for all directory objects
prompt //        dirs permissions should be as strict as possible (depends on the way theese obects are used by running application) 
prompt // Note: by default the following directory objects are created
prompt //             *) owner:SYS   DATA_PUMP_DIR  $ORACLE_HOME/oracle/orahome/rdbms/log/
prompt //             *) owner:SYS   XMLDIR         $ORACLE_HOME/demo/schema/order_entry/
prompt //             *) owner:SYS   MEDIA_DIR      $ORACLE_HOME/demo/schema/product_media/
prompt //             *) owner:SYS   LOG_FILE_DIR   $ORACLE_HOME/demo/schema/log/
prompt //             *) owner:SYS   WORK_DIR       /ade/aime_10.2_lnx_push/oracle/work
prompt //             *) owner:SYS   DATA_FILE_DIR  $ORACLE_HOME/demo/schema/sales_history/
prompt //             *) owner:SYS   SUBDIR         $ORACLE_HOME/demo/schema/order_entry//2002/Sep
prompt //             *) owner:SYS   ADMIN_DIR      /ade/aime_10.2_lnx_push/oracle/md/admin
prompt //       we recommend to keep just DATA_PUMP_DIR and remove the othes
prompt //
col OWNER format A33
col DIRECTORY_NAME format A33
col DIRECTORY_PATH format A66
select * from sys.dba_directories;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [16] DIRECTORY OBJECTS - granted privileges
set termout off
prompt // CHECK: any privileges to DIRECTORY objects should be granted only when required 
prompt //        avoid granting any privileges to PUBLIC
prompt // Note: default privileges to DATA_PUMP_DIR: IMP_FULL_DATABASE, EXP_FULL_DATABASE  - READ and WRITE
prompt //
col OWNER format A20
col GRANTEE format A20
col DIRECTORY format A70
col PRIVILEGE format A15
select D.directory_name || ' - ' || D.directory_path Directory, D.owner, R.grantee, R.privilege
from dba_directories D
left join (select distinct table_name, grantee, privilege from dba_tab_privs) R on R.table_name=D.directory_name; 
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [17] AUDIT CONFIGURATION - general options:
set termout off
prompt // CHECK: review whether auditing subsystem is activated
prompt //           - audit_sys_operations should be TRUE
prompt //           - audit_trail should be OS (preffered) or DB (beware of filling up the tablespace SYSTEM)
prompt //           - audit_file_dest (check filesystem permissions for this directory)
prompt // Note: on MS Windows audit_trail=OS means that the audit records are sent to Eventlog (not into *.aud files in defined directory) 
prompt //
col NAME format A22
col VALUE format A88
select name, value from v$parameter where lower(name) in ('audit_sys_operations', 'audit_file_dest', 'audit_trail','audit_syslog_level');
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [18] AUDIT - default audit options 
set termout off
prompt // CHECK: can be empty
prompt //
select * from all_def_audit_opts;
clear col
 
-- ----------------------------------------------

prompt
set termout on
prompt // [19] AUDIT - privilege auditing
set termout off
prompt // CHECK: for all users should be required auditing of (absolute must):
prompt //           - CREATE SESSION
prompt //           - ALTER SYSTEM / DATABASE 
prompt //           - GRANT ANY PRIVILEGE / ROLE / OBJECT PRIVILEGE 
prompt //        next we recommend to audit also:
prompt //           - EXEMPT ACCESS POLICY / IDENTITY POLICY
prompt //           - CREATE/ALTER/BECOME USER
prompt //           - CREATE LIBRARY
prompt //           - CREATE ANY *
prompt //           - DROP ANY *
prompt //           - SELECT/DELETE/UPDATE ANY DICTIONARY
prompt //           - EXECUTE ANY PROCEDURE
prompt //           - SELECT/DELETE/UPDATE ANY TABLE
prompt //
select user_name, privilege, success, failure from dba_priv_audit_opts;
clear col
 
-- ----------------------------------------------

prompt
set termout on
prompt // [20] AUDIT - statement auditing
set termout off
prompt // CHECK: setting are approx. the same as by "privilege auditing"
prompt //
select user_name, audit_option, success, failure from dba_stmt_audit_opts;
clear col
 
-- ----------------------------------------------

prompt
set termout on
prompt // [21] AUDIT - object audit 
set termout off
prompt // CHECK: at least the following object auditing should be set:
prompt //           - any modifications of AUD$ (when audit_trai=DB)
prompt //           - all operations with USER$, LINK$
prompt //           - all operations with DBA_USERS
prompt //           - other object auditing should be defined in co-operation with application vendor
prompt //
col OWNER format A25
col OBJECT_NAME format A25
col OBJECT_TYPE format A20
select * from dba_obj_audit_opts where
  (ALT not in ('-/-','/')) or (AUD not in ('-/-','/')) or 
  (COM not in ('-/-','/')) or (DEL not in ('-/-','/')) or
  (GRA not in ('-/-','/')) or (IND not in ('-/-','/')) or
  (INS not in ('-/-','/')) or (LOC not in ('-/-','/')) or
  (REN not in ('-/-','/')) or (SEL not in ('-/-','/')) or
  (UPD not in ('-/-','/')) or (REF not in ('-/-','/')) or
  (EXE not in ('-/-','/')) or (CRE not in ('-/-','/')) or
  (REA not in ('-/-','/')) or (WRI not in ('-/-','/')); 
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [22] AUDIT - fine grained auditing policies 
set termout off
prompt // CHECK: can be empty (in most cases the FGA will not be used)
prompt //
select * from dba_audit_policies;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [23] VPD - fine grained security policies 
set termout off
prompt // CHECK: can be empty (using VPD depends on the application)
prompt //
select * from v$vpd_policy;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [24] DB JOBS - scheduler 
set termout off
prompt // CHECK: review "forgotten" jobs possibly suspicious jobs created by common users
prompt //        review jobs created by low-privileged user however executed under privileged account
prompt // Note: default instalation creates one scheduled job - owner SYS, executes "EMD_MAINTENANCE.EXECUTE_EM_DBMS_JOB_PROCS();"
prompt //
col LOG_USER format A20
col PRIV_USER format A20
col WHAT format A66
select job, to_char(next_date,'DD.MM.YY HH24:MI') NEXTRUN, log_user, priv_user, what from dba_jobs;
clear col
  
-- ----------------------------------------------

prompt
set termout on
prompt // [25] DB LINKS - all 
set termout off
prompt // CHECK: database links are generally recommended as a good-practice
prompt //           - especially risky are PUBLIC links with fixed user
prompt //           - USERNAME is recommended to be either empty or set to "CURRENT_USER"
prompt //           - avoid using local links pointing to the database itself 
prompt //
col OWNER format A20
col DB_LINK format A50
col USERNAME format A20
col HOST format A20
select * from DBA_DB_LINKS;
clear col
   
-- ----------------------------------------------

prompt
set termout on
prompt // [26] DB LINKS - details 
set termout off
prompt // CHECK: 
prompt //  - if LEN_PWD or LEN_APWD are not empty - plain text passwors are stored (should not happen on 10gR2+)
prompt //  - if AUTHUSR ii not empty -> SHARED link - not recommended practice
prompt //  - if USERID is not empty and not equal to "CURRENT_USER" it is Fixed User Link - strongly not recommended  
prompt //
col NAME format A45
col HOST format A20
col USERID format A20
col AUTHUSR format A20
col len_pwd format 99
col len_apwd format 99
select name, host, userid, authusr, length(password) len_pwd, length(authpwd) len_apwd from link$;
clear col
  
-- ----------------------------------------------

prompt
set termout on
prompt // [27] DB USERS - statistics 
set termout off
prompt // CHECK: review the following
prompt //           - default tablespace SYSTEM should be set just for SYS
prompt //           - temporaty tablespace should be always set to TEMP (by all accounts)
prompt //           - granted password profile should be compliant with your company password policy
prompt //
select 'Status - ' || Account_Status Item, count(*) Count from dba_users group by Account_status
union
select 'Profile - ' || Profile, count(*) Count from dba_users group by Profile
union
select 'Default tablespace - ' || DEFAULT_TABLESPACE, count(*) Count from dba_users group by DEFAULT_TABLESPACE
union
select 'Temporary tablespace - ' || TEMPORARY_TABLESPACE, count(*) Count from dba_users group by TEMPORARY_TABLESPACE
union
select 'Auth - standard (password)', count(*) Count from dba_users where length(PASSWORD)=16
union
select 'Auth - nonstandard (external, anonymous etc.)', count(*) Count from dba_users where length(PASSWORD)<>16
union
select 'Status - OPEN without expirations', count(*) from dba_users where ACCOUNT_STATUS like '%OPEN%' and (EXPIRY_DATE is NULL or EXPIRY_DATE= '')
union
select 'Total number of DB accounts', count(*) Count from dba_users;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [28] DB USERS - default tablespace SYSTEM 
set termout off
prompt // CHECK: just SYS should have def. TS = SYSTEM - set SYSAUX for others
prompt // Note: by default default tablespace = SYSTEM for SYS, SYSTEM, OUTLN, MGMT_VIEW
prompt //       it means SYSTEM, OUTLN, MGMT_VIEW should be changed 
prompt //
select username,account_status,default_tablespace,created
from dba_users
where default_tablespace='SYSTEM';
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [29] DB USERS - tablespace quotas
set termout off
prompt // just info
prompt //
select tablespace_name,username,decode(max_bytes,-1,'UNLIMITED',max_bytes/(1024*1024) || ' MB') Quota
from dba_ts_quotas order by tablespace_name, username;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [30] DB USERS - database usage (space occupied by user objects) - statistics
set termout off
prompt // just info
prompt //
col SIZE_MB format 9999999.99
select owner,sum(bytes/(1024*1024)) Size_MB from dba_segments
group by owner order by size_mb desc;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [31] DB USERS - database usage (space occupied by user objects) - details
set termout off
prompt // excluded: 'SYS','SYSTEM','DBA','EXFSYS','DMSYS','CTXSYS','SYSMAN','WMSYS','TSMSYS','ORDSYS'
prompt // just info
prompt //
col SIZE_MB format 9999999.99
select owner,segment_type,sum(bytes/(1024*1024)) Size_MB from dba_segments where owner not in ('SYS','SYSTEM','DBA','EXFSYS','DMSYS','CTXSYS','SYSMAN','WMSYS','TSMSYS','ORDSYS')
group by owner, segment_type order by owner;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [32] USER PROFILES - ALL PASSWORD OPTIONS
set termout off
prompt // CHECK: review whether password policy complies with you company security policy
prompt //
select * from dba_profiles where RESOURCE_TYPE='PASSWORD' order by 1,2;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [33] USER PROFILES - PASSWORD_VERIFY_FUNCTION
set termout off
prompt // CHECK: no profiles with PASSWORD_VERIFY_FUNCTION='NULL' should exist
prompt //        password quality should be enforced even for system accounts like SYS, SYSTEM, ...
prompt //
select PROFILE, LIMIT PASSWORD_VERIFY_FUNCTION from dba_profiles where RESOURCE_TYPE='PASSWORD' and RESOURCE_NAME='PASSWORD_VERIFY_FUNCTION';
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [34] DB USERS - ALL:
set termout off
prompt // CHECK: review whether open account are only those necessary for running applications
prompt //
col USERNAME format A20
col ACCOUNT_STATUS format A20
col DEFAULT_TABLESPACE format A20
col TEMPORARY_TABLESPACE format A20
col PROFILE format A20
select USERNAME,ACCOUNT_STATUS,DEFAULT_TABLESPACE,TEMPORARY_TABLESPACE,PROFILE,to_char(CREATED,'DD.MM.YY'),to_char(LOCK_DATE,'DD.MM.YY') LOCKED,to_char(EXPIRY_DATE,'DD.MM.YY') EXPIRED 
from dba_users order by username;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [35] DB USERS - EXPIRED BUT STILL OPEN ACCOUNTS:
set termout off
prompt // summary of expired but still open accounts
prompt // CHECK: the result shoul be emty - if not, lock the detected accounts
prompt //
col USERNAME format A20
col ACCOUNT_STATUS format A20
col DEFAULT_TABLESPACE format A20
col TEMPORARY_TABLESPACE format A20
col PROFILE format A20
select USERNAME,ACCOUNT_STATUS,DEFAULT_TABLESPACE,TEMPORARY_TABLESPACE,PROFILE,EXPIRY_DATE EXPIRED 
from dba_users where expiry_date < sysdate
and ACCOUNT_STATUS not like '%LOCKED%' order by username;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [36] DB USERS - NON-STANDARD AUTH
set termout off
prompt // CHECK: - review whether using non standard authentication is necessary
prompt //        - by default ANONYMOUS: PASSWORD=anonymous, ACCOUNT_STATUS=EXPIRED-LOCKED (acceptable)
prompt //        - when using EXTERNAL autentication ensure that remote_os_authent=FALSE and remote_os_roles=FALSE are set
prompt //        - it is generally recommended to avoid using EXTERNAL authentication if possible 
prompt //
col USERNAME format A20
col PASSWORD format A20
col ACCOUNT_STATUS format A20
col DEFAULT_TABLESPACE format A20
col TEMPORARY_TABLESPACE format A20
col PROFILE format A20
select USERNAME,PASSWORD,ACCOUNT_STATUS,DEFAULT_TABLESPACE,TEMPORARY_TABLESPACE,PROFILE 
from dba_users where length(PASSWORD)<>16 order by username;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [37] DB USERS WITH SIMPLE PASSWORD
set termout off
prompt // CHECK: change password lock any open accounts with simple password
prompt //
@oracheck-defpasswd.sql
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [38] DB USERS - SYSDBA / SYSOP
set termout off
prompt // CHECK: just SYS should be granted SYSOP and SYSDBA
prompt //
select USERNAME, SYSDBA, SYSOPER from v$pwfile_users;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [39] DB ROLES - statistics 
set termout off
prompt // just info
prompt //
select p.grantee, nvl(u.account_status,'<role not an account>') account_status, p.roles_count from 
(select grantee, count(granted_role) roles_count from dba_role_privs group by grantee) p
left join dba_users u on p.grantee=u.username
order by grantee;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [40] DB ROLES - DBA and other strong roles
set termout off
prompt // CHECK: - DBA only SYS, SYSMAN, SYSTEM (possibly special DB accounts od DB admins)
prompt //        - EXP_FULL_DATABASE only DBA, SYS
prompt //        - IMP_FULL_DATABASE only DBA, SYS
prompt //        - OLAP_DBA only DBA, SYS, OLAPSYS
prompt //
select P.GRANTEE, nvl(u.account_status,'<role not an account>') account_status, P.granted_role from (SELECT grantee, granted_role FROM dba_role_privs where upper(granted_role) in ('DBA','OLAP_DBA','DBA','OLAP_DBA','IMP_FULL_DATABASE','EXP_FULL_DATABASE')) P 
LEFT JOIN dba_users U ON P.grantee=U.username
order by granted_role;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [41] DB ROLES - catalog related roles 
set termout off
prompt // excluded: 'SYS','SYSTEM','SYSMAN','DBA','OLAP_DBA','IMP_FULL_DATABASE','EXP_FULL_DATABASE'
prompt // CHECK: - the result should be empty
prompt //        - avoid granting *CATALOG* roles to any application roles/accounts 
prompt // Note: SELECT_CATALOG_ROLE allows to query (besides others) DBA_USERS view (contains sensitive user password hashes)
prompt //
select P.GRANTEE, nvl(u.account_status,'<role not an account>') account_status, P.granted_role from 
(
  SELECT grantee, granted_role FROM dba_role_privs where 
	granted_role like '%CATALOG%' and
	grantee not in ('SYS','SYSTEM','SYSMAN','DBA','OLAP_DBA','IMP_FULL_DATABASE','EXP_FULL_DATABASE')
) P 
LEFT JOIN dba_users U ON P.grantee=U.username
order by granted_role;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [42] DB ROLES - roles either protected by password of not set by DEFAULT  
set termout off
prompt // overview of DB roles that 
prompt // - are granted with option DEFAULT=NO (calling SET ROLE is required for their activation)
prompt // - or are protected by password (SET ROLE reqires password)
prompt // CHECK: - in case of DEFAULT_ROLE=no + PASSWORD_REQUIRED be suspicous about "security by obscurity"
prompt //        - using roles with PASSWORD_REQUIRED should be assessed according to application architecture
prompt //
col default_role format A18
col password_required format A18
select nvl(p.grantee,'<role not granted>') grantee,r.role,nvl(p.default_role,'<role not granted>') default_role,r.password_required,nvl(s.privilege,'<no sysprivs granted>') system_priv_granted_to_role
from dba_role_privs p, dba_sys_privs s, dba_roles r
where r.role = s.grantee (+) 
and p.granted_role (+) = r.role
and (p.default_role='NO' or r.password_required='YES')
order by p.grantee, p.granted_role, r.password_required, s.privilege;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [43] DB ROLES - roles granted with ADMIN_OPTION  
set termout off
prompt // excluded: 'SYS','SYSTEM','DBA','SYSMAN','CTXSYS','WMSYS'
prompt // CHECK: the result should be empty
prompt //
select * from dba_role_privs where ADMIN_OPTION='YES' and GRANTEE not in ('SYS','SYSTEM','DBA','SYSMAN','CTXSYS','WMSYS') order by grantee;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [44] DB SYSPRIVS - statistics 
set termout off
prompt // just info
prompt // CHECK: review especially number of granted sysprivs by application accounts/roles 
prompt //
select P.GRANTEE, nvl(u.account_status,'<role not an account>') account_status, P.sysprivs_count from 
(SELECT grantee, count(privilege) sysprivs_count FROM dba_sys_privs group by grantee) P
LEFT JOIN dba_users U ON P.grantee=U.username
order by 1;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [45] DB SYSPRIVS - strong system privileges
set termout off
prompt // excluded - system account: 'SYS','SYSTEM','SYSMAN'
prompt // excluded - strong DB roles: 'DBA','OLAP_DBA','IMP_FULL_DATABASE','EXP_FULL_DATABASE'
prompt // strong privileges = '%ANY%','%ALTER%','%BECOME%','%EXEMPT%','CREATE LIBRARY','CREATE SESSION',
prompt //                     '%ADMINISTER%','%GRANT%' (excluded 'CREATE SESSION','ALTER SESSION') 
prompt // CHECK: review the following
prompt //           - ALTER USER, BECOME_USER, CREATE LIBRARY, CREATE ANY LIBRARY, * ANY DICTIONARY, GRANT ANY * 
prompt //           - ALTER SYSTEM, ALTER DATABASE, EXEMPT ACCESS POLICY, EXEMPT IDENTITY POLICY 
prompt //           - SELECT ANY TABLE, DELETE ANY TABLE, UPDATE ANY TABLE, EXECUTE ANY PROCEDURE, DROP ANY *
prompt //           - it is not neccessary to deal with locked account 
prompt // Note: sysprivs granted by default in 10gR2 (all are acceptable)
prompt // - AQ_ADMINISTRATOR_ROLE MANAGE ANY QUEUE
prompt // - AQ_ADMINISTRATOR_ROLE DEQUEUE ANY QUEUE
prompt // - AQ_ADMINISTRATOR_ROLE ENQUEUE ANY QUEUE
prompt // - JAVADEBUGPRIV DEBUG ANY PROCEDURE
prompt // - OEM_MONITOR SELECT ANY DICTIONARY
prompt // - OEM_MONITOR MANAGE ANY QUEUE
prompt // - OEM_MONITOR ANALYZE ANY
prompt // - OEM_MONITOR ANALYZE ANY DICTIONARY
prompt // - OEM_ADVISOR ADMINISTER SQL TUNING SET
prompt // - SCHEDULER_ADMIN EXECUTE ANY PROGRAM
prompt // - SCHEDULER_ADMIN CREATE ANY JOB
prompt // - SCHEDULER_ADMIN EXECUTE ANY CLASS
prompt // - DBSNMP SELECT ANY DICTIONARY 
prompt // - OUTLN EXECUTE ANY PROCEDURE
prompt // - all other are granted to locked account
prompt //
select P.grantee,nvl(u.account_status,'<role not an account>') account_status,P.privilege,P.ADMIN_OPTION from dba_sys_privs P 
left join dba_users U on U.username=P.grantee
where 
P.privilege in (select name from SYSTEM_PRIVILEGE_MAP where name like '%ANY%' or name like '%ALTER%' or name like '%BECOME%' or name like '%EXEMPT%' or name='CREATE LIBRARY' or name='CREATE SESSION' or name like '%ADMINISTER%' or name like '%GRANT%') and
P.privilege not in ('CREATE SESSION','ALTER SESSION') and P.grantee not in ('SYS','SYSTEM','SYSMAN','DBA','OLAP_DBA','IMP_FULL_DATABASE','EXP_FULL_DATABASE')
order by account_status,grantee,privilege; 
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [46] DB SYSPRIVS - privileges granted "WITH ADMIN OPTION"
set termout off
prompt // excluded: 'SYS','SYSTEM','DBA','SCHEDULER_ADMIN','AQ_ADMINISTRATOR_ROLE'
prompt // CHECK: the result should be empty
prompt //
select p.grantee, nvl(u.account_status,'<role not an account>') account_status, p.privilege from 
(select grantee, privilege from dba_sys_privs where ADMIN_OPTION='YES' and GRANTEE not in ('SYS','SYSTEM','DBA','SCHEDULER_ADMIN','AQ_ADMINISTRATOR_ROLE')) P
left join dba_users u on p.grantee=u.username
order by grantee,privilege;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [47] DB SYSPRIVS - privileges granted directly to a DB account (not to a role)
set termout off
prompt // excluded: 'SYS','SYSTEM','DBA','EXFSYS','DMSYS','CTXSYS','SYSMAN','WMSYS','TSMSYS','ORDSYS','ORDPLUGINS','OUTLN','DBSNMP','MDSYS','OLAPSYS'
prompt // granting sysprivs directly to DB accounts (not through DB roles) does not comply with  best-practice
prompt // CHECK: acceptable exceptions (all granted by default installation):
prompt // - ANONYMOUS          CREATE SESSION
prompt // - DIP                CREATE SESSION 
prompt // - MDDATA             UNLIMITED TABLESPACE
prompt // - SI_INFORMTN_SCHEMA UNLIMITED TABLESPACE
prompt //
select * from dba_sys_privs where 
GRANTEE not in ('SYS','SYSTEM','DBA','EXFSYS','DMSYS','CTXSYS','SYSMAN','WMSYS','TSMSYS','ORDSYS','ORDPLUGINS','OUTLN','DBSNMP','MDSYS','OLAPSYS') and 
GRANTEE in (select username from dba_users)
order by grantee;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [48] DB OBJECTS OWNERS - statistics 
set termout off
prompt // just info
prompt // CHECK: be suspicious about users owning just a few DB objects
prompt //
SELECT owner, count(*) object_count FROM dba_objects group by owner order by owner;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [49] DB OBJECTS OWNERS - statistics/details 
set termout off
prompt // excluded: 'SYS','SYSTEM','DBA','EXFSYS','DMSYS','CTXSYS','SYSMAN','WMSYS','TSMSYS','XDB',
prompt //           'ORDSYS','ORDPLUGINS','PUBLIC','SI_INFORMTN_SCHEMA','OUTLN','DBSNMP','MDSYS','OLAPSYS'
prompt // CHECK: - the result should contain just application objects
prompt //        - recommendation: application obects should be owned by special accounts that are not used for day-to-day operations
prompt //
select owner, object_type, count(*) count 
from dba_objects
where owner not in ('SYS','SYSTEM','DBA','EXFSYS','DMSYS','CTXSYS','SYSMAN','WMSYS','TSMSYS','XDB','ORDSYS','ORDPLUGINS','PUBLIC','SI_INFORMTN_SCHEMA','OUTLN','DBSNMP','MDSYS','OLAPSYS')
group by owner, object_type
order by owner;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [50] DB OBJECTS - TOP20 largest DB objects 
set termout off
prompt // just info
prompt //
col segment_name format A40
col segment_type format A20
col SIZE_MB format 9999999
select * from 
(
  select owner,segment_name,segment_type,tablespace_name,bytes/(1024*1024) Size_MB         
  from dba_segments
  order by bytes desc
) where rownum<=20; 
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [51] DB OBJPRIVS - statistics 
set termout off
prompt // basic statistics of granted object privileges
prompt // just info
prompt //
select p.grantee, nvl(u.account_status,'<role not an account>') account_status, p.privilege, p.count from 
(select grantee, privilege, count(*) Count from dba_tab_privs group by grantee, privilege order by grantee) p
left join dba_users u on p.grantee=u.username
order by grantee;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [52] DB OBJPRIVS - privileges granted directly to a DB account (not to a role)
set termout off
prompt // excluded: 'SYS','SYSTEM','DBA','EXFSYS','DMSYS','CTXSYS','SYSMAN','WMSYS','TSMSYS','ORDSYS','ORDPLUGINS','DBSNMP','OUTLN','MGMT_VIEW','MDSYS','OLAPSYS','XDB'
prompt // granting object privileges directly to DB accounts (not through DB roles) does not comply with  best-practice
prompt // CHECK: the result should be empty
prompt //
select OWNER object_owner,TABLE_NAME object_name,GRANTEE,PRIVILEGE from dba_tab_privs where 
GRANTEE not in ('SYS','SYSTEM','DBA','EXFSYS','DMSYS','CTXSYS','SYSMAN','WMSYS','TSMSYS','ORDSYS','ORDPLUGINS','DBSNMP','OUTLN','MGMT_VIEW','MDSYS','OLAPSYS','XDB') and 
GRANTEE in (select username from dba_users)
order by grantee;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [53] DB OBJPRIVS - privileges granted as GRANTABLE  
set termout off
prompt // excluded: 'SYS','SYSTEM','DBA','SYSMAN','CTXSYS','WMSYS','ORDSYS','OLAPSYS','MDSYS','PUBLIC'
prompt // CHECK: the result should be empty
prompt //
select GRANTEE, PRIVILEGE, count(*) Count FROM dba_tab_privs where GRANTABLE='YES' and grantee not in ('SYS','SYSTEM','DBA','SYSMAN','CTXSYS','WMSYS','ORDSYS','OLAPSYS','MDSYS','PUBLIC') group by GRANTEE, PRIVILEGE;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [54] DB OBJECTS - EXTERNAL LIBRARIES
set termout off
prompt // CHECK: - external libraries are risky by design - it is definitely better to avoid them
prompt //        - in all cases review the filesystem permissions to dynamic libraries (*.dll, *.so)
prompt //          (overit zapisovatelnost pouze uzivatelem, pod kterym je provozovana databaze)
prompt //        - the following ext. libraries are installed by default (in 10gR2) - all are acceptable
prompt //            *)DBMS_SUMADV_LIB -> $ORACLE_HOME/lib/libqsmashr.so
prompt //            *)ORDIMLIBS -> $ORACLE_HOME/lib/libordim10.so
prompt //        - other default ext. libraries (samples/demo apps) should be removed
prompt //        - if ext. libraries are not used at all it is recommended to completely disable PlsExtProc in oracle listener
prompt //
col STATUS format A8
col OWNER format A15
col LIBRARY_NAME format A25
col FILE_SPEC format A60
select * from dba_libraries where file_spec is not null;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [55] DB OBJECTS - packages using EXTERNAL LIBRARIES
set termout off
prompt // list of DB packages using external libraries 
prompt // CHECK: disable/remove all ext. libraries that are not used in any DB package  
prompt // Note: default ext. libraries (Oracle 10gR2) are used from the following packages
prompt // - DBMS_SUMADV_LIB from package DBMS_SUMADV
prompt // - ORDIMLIBS from packages ORDIMGEXTCODEC_PKG and ORDIMG_PKG 
prompt //
col powner format A15
col library_owner format A15
col pname format A24
col library_name format A25
col file_spec format A60
select l.owner library_owner, l.library_name, l.file_spec, d.owner powner, d.name pname, d.type
from dba_libraries l, dba_dependencies d
where l.file_spec is not null and d.referenced_owner(+) = l.owner and d.referenced_name(+) = l.library_name;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [56] DB OBJECTS - packages using EXTERNAL LIBRARIES - source code
set termout off
prompt // source code of DB packages using external libraries
prompt // CHECK: - standard packages DBMS_SUMADV, ORDIMGEXTCODEC_PKG, ORDIMG_PKG are WRAPPED (nothing to check) 
prompt //        - other packages should be reviewed (depending on lib purpouse in specific application)
prompt //        - the arising risks should be discussed with application vendor 
prompt //
col NAME format A35
col SOURCE_CODE format A100
col LINE noprint
break on name
select name,line,text source_code from (
  (
    select name || '-' || type name,line,text from dba_source
    where (owner,name) in
    (
      select d.owner downer, d.name dname from dba_libraries l, dba_dependencies d
      where l.file_spec is not null and d.referenced_owner(+) = l.owner and d.referenced_name(+) = l.library_name
    )
    and (owner,name,type) not in (select distinct owner,name,type from dba_source where lower(text) like 'package % wrapped%')
  )
union
  (
    select name || '-' || type name,1,'Wrapped !!!' text from dba_source
    where (owner,name) in
    (
      select d.owner downer, d.name dname from dba_libraries l, dba_dependencies d
      where l.file_spec is not null and d.referenced_owner(+) = l.owner and d.referenced_name(+) = l.library_name
    )
    and (owner,name,type) in (select distinct owner,name,type from dba_source where lower(text) like 'package % wrapped%')
  )
) order by name,line;
clear breaks
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [57] DB OBJECTS - TRIGGERS (startup, logon)
set termout off
prompt // startup and logon triggers are executed by every DB startup / every user login
prompt // these triggers are quite significant from security point of view
prompt // CHECK: - by default the following triggers are installed (both are acceptable):
prompt //                 *) AURORA$SERVER$STARTUP - owner:SYS - AFTER EVENT - STARTUP - status:DISABLED
prompt //                 *) OLAPISTARTUPTRIGGER - owner:SYS - AFTER EVENT - STARTUP - status:ENABLED
prompt //        - check for any other triggers
prompt //
col TRIGGERING_EVENT format A30
select table_owner owner,trigger_name,trigger_type,triggering_event,status
from dba_triggers
where triggering_event like '%STARTUP%' or triggering_event like '%LOGON%'
order by table_owner, trigger_name;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [58] DB OBJECTS - TRIGGERS (startup, logon) - source code
set termout off
prompt // CHECK: - review any non-standard triggers (others than AURORA$SERVER$STARTUP, OLAPISTARTUPTRIGGER)
prompt //        - especially analyze carefully LOGON trigger
prompt //        - it is strongly recommended to avoid using wrapped LOGON trigger
prompt //
col NAME format A25
col SOURCE_CODE format A100
col LINE noprint
break on name
select name,line,source_code from (
  (
    select name,line,text source_code
    from dba_source
    where (owner,name) in (select owner,trigger_name from dba_triggers where triggering_event like '%STARTUP%' or triggering_event like '%LOGON%')
    and (owner,name,type) not in (select distinct owner,name,type from dba_source where lower(text) like 'package % wrapped%')
  )
union
  (
    select name,1,'Wrapped !!!' source_code
    from dba_source
    where (owner,name) in (select owner,trigger_name from dba_triggers where triggering_event like '%STARTUP%' or triggering_event like '%LOGON%')
    and (owner,name,type) in (select distinct owner,name,type from dba_source where lower(text) like 'package % wrapped%')
  )
) order by name,line;
clear breaks
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [59] DB OBJECTS - JAVA OBJECTS - statistics
set termout off
prompt // just info
prompt //
select owner,object_type,count(*) count from dba_objects where object_type like '%JAVA%' group by owner,object_type order by owner,object_type;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [60] DB OBJECTS - JAVA PERMISSIONS - complete list
set termout off
prompt // just info
prompt //
col TYPE_NAME format A50
col NAME format A55
col KIND format A8
col GRANTEE format A15
col ACTION format A30
select kind,grantee,type_name,action,enabled,name from dba_java_policy order by grantee;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [61] DB OBJECTS - JAVA PERMISSION - java.io.FilePermission
set termout off
prompt // java.io.FilePermission - extremely risky by design - allows file operations on OS level like read,write,execute,delete 
prompt // java.net.SocketPermission - allows create TCP connections
prompt // CHECK: - restrict usage of "java.io.FilePermission"
prompt //        - java.io.FilePermission by default only 
prompt //              *) JAVASYSPRIV  <<ALL FILES>>     read,write,execute,delete
prompt //              *) JAVAUSERPRIV <<ALL FILES>>     read
prompt //              *) JAVA_DEPLOY  bin/chmod         execute
prompt //              *) JAVA_DEPLOY  javavm/admin/*    write
prompt //              *) JAVA_DEPLOY  javavm/deploy/*   read
prompt //              *) MDSYS        md/lib/*          read                      
prompt //        - java.net.SocketPermission
prompt //              *) EJBCLIENT     *                connect,resolve
prompt //              *) JAVASYSPRIV   *                accept,connect,listen,resolve
prompt //              *) JAVAUSERPRIV  *                connect,resolve
prompt //        - any other cases sould be carefully substantiated
prompt //
-- /**/
col NAME format A35
col KIND format A8
col GRANTEE format A15
col ACTION format A30
col TYPE_NAME format A30
select kind,grantee,type_name,action,name,enabled 
from dba_java_policy 
where lower(type_name) in ('java.io.filepermission','java.net.socketpermission')
order by type_name,grantee;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [62] INVALID OBJECTS - statistics
set termout off
prompt // CHECK: there should be no INVALID objects (repair them with rdbms/admin/utlrp.sql)
prompt //
select Owner,Object_Type,Status,count(*) count  
from DBA_OBJECTS where Status != 'VALID' 
group by Owner,Object_Type,Status
order by Owner,Object_Type;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [63] INVALID OBJECTS - details
set termout off
prompt // CHECK: there should be no INVALID objects
prompt //
col OBJECT_NAME format A35
col OBJECT_TYPE format A15
select Owner,Object_Type,Object_Name,Status  
from DBA_OBJECTS
where Status != 'VALID'
order by 1,3,2;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [64] SENSITIVE OBJECTS - access rights
set termout off
prompt // we check just: 'AUD$','USER_HISTORY$','LINK$','USER$','SOURCE$','STATS$SQLTEXT','STATS$SQL_SUMMARY','DBA_USERS','DBA_SOURCE'
prompt // CHECK: - by default (in 10gR2)
prompt //              *) AUD$         priv:DELETE granted_to:DELETE_CATALOG_ROLE
prompt //              *) DBA_SOURCE   SELECT granted_to:SELECT_CATALOG_ROLE
prompt //              *) DBA_USERS    SELECT granted_to:CTXSYS, SELECT_CATALOG_ROLE
prompt //              *) USER$        SELECT granted_to:CTXSYS, OLAPSYS, XDB
prompt //        - any other should be substantiated
prompt //        - you do not neeed to concern about locked accounts
prompt //
col OWNER format A15
col GRANTEE format A25
col GRANTEE_SATUS format A25
col TABLE_NAME format A20
col PRIVILEGE format A20
select owner,table_name,privilege,grantee,nvl(account_status,'<role not an account>') grantee_satus
from dba_tab_privs p,dba_users u 
where p.grantee = u.username (+) 
and table_name in ('AUD$','USER_HISTORY$','LINK$','USER$','SOURCE$','STATS$SQLTEXT','STATS$SQL_SUMMARY','DBA_USERS','DBA_SOURCE')
order by TABLE_NAME;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [65] SENSITIVE OBJECTS - access rights (special privileges)
set termout off
prompt // excluded: 'SYS','SYSTEM','DBA','SYSMAN','EXP_FULL_DATABASE','IMP_FULL_DATABASE'
prompt // CHECK: acceptable is at most
prompt // - DBSNMP       Priv: SELECT ANY DICTIONARY
prompt // - OEM_MONITOR  Priv: SELECT ANY DICTIONARY
prompt // - OLAP_DBA     Priv: SELECT ANY DICTIONARY
prompt // - OLAP_USER    Role: SELECT_CATALOG_ROLE
prompt // - OLAP_DBA     Role: SELECT_CATALOG_ROLE
prompt //
select * from
(
  select grantee, 'Priv: ' || privilege priv_role from dba_sys_privs where privilege='SELECT ANY DICTIONARY'
  union
  select grantee, 'Role: ' || granted_role priv_role from dba_role_privs where granted_role like '%CATALOG%'
)
where grantee not in ('SYS','SYSTEM','DBA','SYSMAN','EXP_FULL_DATABASE','IMP_FULL_DATABASE')
order by priv_role;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [66] SENSITIVE OBJECTS - modifications
set termout off
prompt // we check: 'AUD$','USER_HISTORY$','LINK$','USER$','SOURCE$','STATS$SQLTEXT','STATS$SQL_SUMMARY','DBA_USERS','DBA_SOURCE'
prompt // CHECK: - these sensitive objects should be modified just during DB installation or patchset/CPU instalation
prompt //        - check whether modification dates/times correspond with installations dates/times
prompt //
col OWNER format A20
col OBJECT_NAME format A20
col OBJECT_TYPE format A20
select owner,object_name,object_type,to_char(created,'DD.MM.YYYY HH24:MI') Created,to_char(LAST_DDL_TIME,'DD.MM.YYYY HH24:MI') Last_Modified,status
from dba_objects 
where object_name in ('AUD$','USER_HISTORY$','LINK$','USER$','SOURCE$','STATS$SQLTEXT','STATS$SQL_SUMMARY','DBA_USERS','DBA_SOURCE')
order by last_ddl_time desc;
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [67] PUBLIC - privileges/ownership summary 
set termout off
prompt // summary of PUBLIC object ownership / object privileges
prompt // just info
prompt //
select privilege, '-' Detail, count(*) Count
from dba_tab_privs
where grantee='PUBLIC'
group by privilege
UNION
select '(ownership)' privilege, object_type Detail, count(*) countPocet
from dba_objects
where owner='PUBLIC'
group by object_type; 
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [68] PUBLIC - object ownership 
set termout off
prompt // overview of all objects owned by PUBLIC (apart from SYNONYMs)
prompt // CHECK: the result should be empty
prompt //
col OBJECT_NAME format A80
select owner, object_type, object_name
from dba_objects
where object_type <> 'SYNONYM' and owner='PUBLIC';
clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [69] PUBLIC - privileges DELETE/UPDATE/INSERT/ALTER
set termout off
prompt // statistics of DELETE/UPDATE/INSERT/ALTER privileges granted to PUBLIC
prompt // CHECK: - by default just few objects in CTXSYS, WMSYS, XDB schemas
prompt //          slightly more (tens) objects in SYS, SYSTEM, MDSYS, OLAPSYS schemas
prompt //        - review any additional privileges (and revoke if possible)
prompt //        - especially avoid granting PUBLIC any modification privileges to applications objects
prompt //
select grantee, privilege, owner object_owner, count(table_name) object_Count
from dba_tab_privs
where grantee='PUBLIC' and privilege in ('DELETE','UPDATE','INSERT','ALTER')
group by grantee, privilege, owner
order by owner;
clear col

-- DETAILS:
-- select grantee, privilege, owner object_owner, table_name
-- from dba_tab_privs
-- where grantee='PUBLIC' and privilege in ('DELETE','UPDATE','INSERT','ALTER')
-- order by owner;
-- clear col

-- ----------------------------------------------

prompt
set termout on
prompt // [70] PUBLIC - privileges to special objects 
set termout off
prompt // we check: 'AUD$','USER_HISTORY$','LINK$','USER$','SOURCE$','STATS$SQLTEXT','STATS$SQL_SUMMARY','DBMS_LOB','DBMS_SYS_SQL','DBMS_BACKUP_RESTORE','DBMS_JOB','ALL_USERS','ALL_SOURCE'
prompt // and all objects with names starting 'X$', 'X_$', 'DBA_' or 'UTL_'
prompt // CHECK: - strongly recommended to revode all PUBLIC rights to UTL_FILE, UTL_HTTP, UTL_SMTP, UTL_TCP
prompt //        - DBA_* - we can accept just SELECT for 
prompt //            DBA_AUTO_SEGADV_CTL, DBA_AUTO_SEGADV_SUMMARY, DBA_SDO_MAPS, DBA_SDO_STYLES, DBA_SDO_THEMES
prompt //        - DBMS_* - we can accept just EXECUTE on DBMS_JOB and DBMS_LOB 
prompt //
select table_name, privilege
from dba_tab_privs P
where (P.grantee='PUBLIC')
and
(
  P.table_name IN ('AUD$','USER_HISTORY$','LINK$','USER$','SOURCE$','STATS$SQLTEXT','STATS$SQL_SUMMARY','DBMS_LOB','DBMS_SYS_SQL','DBMS_BACKUP_RESTORE','DBMS_JOB','ALL_USERS','ALL_SOURCE')
  or
  P.table_name like 'X$%' or P.table_name like 'X_$%' OR P.table_name like 'DBA_%' OR P.table_name like 'UTL_%'
)
order by table_name, privilege;
clear col

-- ----------------------------------------------

set termout on
prompt // ORACLE CONFIGURATION CHECK FINISHED
timing stop
prompt // results saved to '&my_outfile'

spool off

quit
